<?
/********************************************************************
 * NotFTP 
 * Copyright (c) 2002, 2003 by Ryan Grove <ryan@wonko.com>
 * All rights reserved.
 *
 * This script is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA  02111-1307  USA
 ********************************************************************/

set_time_limit(TIMEOUT);

// Set style (cookie persists for one year)
if (isset($_POST["style"]))
{
	setcookie("notftpstyle", $_POST["style"], time() + 31536000);
	$newstyle = $_POST["style"];
}

// Set language (cookie persists for one year)
if (isset($_POST["language"]))
{
	setcookie("notftplang", $_POST["language"], time() + 31536000);
	$newlang = $_POST["language"];
}

require_once("functions.php");

if (isset($_POST["address"]))
{
    $address = trim($_POST["address"]);
    $user    = trim($_POST["user"]);
    $pass    = $_POST["pass"];
    
    if (ALLOWUSERPORT === true)
    {
		$port = $_POST["port"];
	}
	else
	{
		$port = DEFAULTPORT;
	}
	
	if (isset($_POST["ssl"]))
		$ssl = true;
	else
		$ssl = false;
}
else
{
    $address = $_COOKIE["notftpaddress"];
    $user    = $_COOKIE["notftpuser"];
    $pass    = $_COOKIE["notftppass"];

    if (ALLOWUSERPORT === true)
    {
		$port = $_COOKIE["notftpport"];
	}
	else
	{
		$port = DEFAULTPORT;
	}
	
	$ssl = $_COOKIE["ssl"];
}

$error = false;

// Should we only allow certain servers?
if (USESERVERLIST)
{
	// Make sure this address is in the list
	$inlist = false;
	foreach($servers as $server)
	{
		if ($address == $server[0])
		{
			$inlist = true;
			$port = $server[1];
		}
	}
	
	// Address wasn't in the list
	if ($inlist === false)
	{
		$error = true;
		$errmsg = $text['errconnect']." $address";
	}
}
else
{
	// If no address is specified, use localhost
	if ($address == "")
	{
		$address = "localhost";
	}
}

// Is this a LAN server? Should we allow it?
if (ALLOWLANSERVER === false)
{
	if (ereg("192\.168\.", $address) || ereg("192\.168\.", gethostbyname($address)) || ereg("192\.168\.", long2ip($address)))
	{
		// Address is internal, do not allow connection
		$error = true;
		$errmsg = $text['errconnect']." $address";
	}
}

if ($user == "")
    $user = "anonymous";

if ($pass == "")
    $pass = "anon@anon.com";

// Try to connect
if ($error === false)
{
    $ssl ? $conn = ftp_connect($address, $port) : $conn = ftp_ssl_connect($address, $port);

    if ($conn === false)
    {
        $error = true;
        $errmsg = $text['errconnect']." $address";
    }

    // Try to login
    if ($error === false)
    {
        $login = @ftp_login($conn, $user, $pass);

        if ($login === false)
        {
            $error = true;
            $errmsg = $text['errlogin']." '$user'";
        }

        // Set login cookies
        if ($error === false)
        {
            setcookie("notftpaddress", $address, 0);
            setcookie("notftpuser", $user, 0);
            setcookie("notftppass", $pass, 0);
            setcookie("notftpport", $port, 0);
            setcookie("notftpssl", $ssl, 0);

            if (TRANSFERLIMIT > 0 && !isset($_COOKIE["transfers"]))
                setcookie("transfers", 0, time() + 86400);

            if (TRANSFERQUOTA > 0 && !isset($_COOKIE["quota"]))
                setcookie("quota", 0, time() + 86400);
        }
    }
}

if ($error)
{
    include_once("lib/header.php");
    echo "<div class=\"error\">".$text['error']." $errmsg </div>\n";
}
else
{
	// If PASVMODE is true, try to activate passive mode
	if (PASVMODE === true)
		@ftp_pasv($conn, true);
	else
		@ftp_pasv($conn, false);

	$rawlist = array();
	$nlist   = array();

	// Change directory
	if (isset($_GET["d"]))
	{
		if (!@ftp_chdir($conn, stripslashes($_GET["d"])))
		{
			$error  = true;
			$errmsg = $text['errcd']." ".str_replace("//", "/", $_GET["d"]);
		}
	}

    // Handle commands
    if (isset($_GET["op"]))
        $op = $_GET["op"];
    elseif (isset($_POST["op"]))
        $op = $_POST["op"];

    if (isset($op))
    {
        switch($op)
        {
            case "cd":
                if (!@ftp_chdir($conn, stripslashes($_POST["cmd"])))
                {
					$error	= true;
					$errmsg	= $text['errcd']." ".str_replace("//", "/", $_POST["cmd"]);
				}
            break;

            case "del":
                $pwd = $_COOKIE["pwd"];

                if (isset($_GET["dir"]))
                {
                    if (!@ftp_rmdir($conn, $_GET["dir"]))
                    {
                    	$error 	= true;
                    	$errmsg	= $text['errrmdir']." ".str_replace("//", "/", $_GET["dir"]);
					}
                }
                elseif (isset($_GET["file"]))
                {
                    if (!@ftp_delete($conn, $pwd."/".$_GET["file"]))
                    {
						$error	= true;
						$errmsg	= $text['errdel']." ".str_replace("//", "/", $_GET["file"]);
					}
                }
                ftp_chdir($conn, $pwd);
            break;

            case "mkdir":
                $pwd    = $_COOKIE["pwd"];
                $newdir = $pwd."/".$_POST["cmd"];
                if (!@ftp_mkdir($conn, $newdir))
                {
					$error	= true;
					$errmsg	= $text['errmkdir']." ".str_replace("//", "/", $_POST["cmd"]);
				}
                ftp_chdir($conn, $pwd);
            break;

            case "logoff":
                // Delete all files downloaded during this session
                $downloaded = unserialize(base64_decode($_COOKIE["files"]));

                if (is_array($downloaded) && count($downloaded) > 0)
                {
                    foreach($downloaded as $file)
                        @unlink(FILEPATH."/$file");
                }

                // Unset all cookies
                setcookie("notftpaddress", "", time() - 3600);
                setcookie("notftpuser", "", time() - 3600);
                setcookie("notftppass", "", time() - 3600);
                setcookie("notftpssl", "", time() - 3600);
                setcookie("pwd", "", time() - 3600);

                // Redirect to front page
                header("Location: ./");
            break;
        }
    }

    // Upload file
    if(isset($_POST["action"]))
    {
        $pwd      = $_COOKIE["pwd"];
        $filename = $_FILES["uploadfile"]["tmp_name"];
        $size     = $_FILES["uploadfile"]["size"];

        if ((TRANSFERLIMIT > 0) && ($_COOKIE["transfers"] > TRANSFERLIMIT))
        {
            $error	= true;
            $errmsg	= $text['errtransfer'];
        }
        else
        {
            if ((SIZELIMIT > 0 && $size > SIZELIMIT))
            {
                $error	= true;
                $errmsg	= $text['errsize'];
            }
            elseif ((TRANSFERQUOTA > 0) && (($_COOKIE["quota"] + $size) > TRANSFERQUOTA))
            {
                $error	= true;
                $errmsg	= $text['errquota'];
            }
            else
            {
                if (!ftp_put($conn, $pwd."/".$_FILES["uploadfile"]["name"], $filename, FTP_BINARY))
                {
                    $error	= true;
                    $errmsg	= $text['errupload'];
				}
                elseif (TRANSFERLIMIT > 0)
                {
                    setcookie("transfers", $_COOKIE["transfers"] + 1, time() + 86400);
				}
            }
        }

        ftp_chdir($conn, $pwd);
    }

    // Download file
    if (isset($_GET["f"]))
    {
        if ((TRANSFERLIMIT > 0) && ($_COOKIE["transfers"] > TRANSFERLIMIT))
        {
            $error	= true;
            $errmsg	= $text['errtransfer'];
        }
        else
        {
			// Try to get file size
			$size = ftp_size($conn, $_GET["d"]."/".$_GET["f"]);

			if ((SIZELIMIT > 0 && $size > SIZELIMIT))
            {
				$error	= true;
				$errmsg	= $text['errsize'];
            }
            elseif ((TRANSFERQUOTA > 0) && (($_COOKIE["quota"] + $size) > TRANSFERQUOTA))
            {
                $error	= true;
                $errmsg	= $text['errquota'];
            }
            elseif ($size != -1)
            {
				// Add this file to our "downloaded" list
                $downloaded = unserialize(base64_decode($_COOKIE["files"]));

                if (is_array($downloaded))
                    array_push($downloaded, $_GET["f"]);
                else
                    $downloaded = array($_GET["f"]);

                $downloaded = array_unique($downloaded);
                setcookie("transfers", $_COOKIE["transfers"] + 1, time() + 86400);
                setcookie("quota", $_COOKIE["quota"] + $size, time() + 86400);
				
                // Download the file
                if (PASSTHRUDOWNLOAD === true)
                {
					// Close the connection so we don't have multiple connections open at once.
					@ftp_quit($conn);
					
                    if ($ssl)
                    {
						if (strtolower($user) == "anonymous")
							$fp = @fopen("ftps://$address:$port".$_GET["d"]."/".$_GET["f"], "rb");
						else
							$fp = @fopen("ftps://$user:$pass@$address:$port".$_GET["d"]."/".$_GET["f"], "rb");
					}
					else
					{
						if (strtolower($user) == "anonymous")
							$fp = @fopen("ftp://$address:$port".$_GET["d"]."/".$_GET["f"], "rb");
						else
							$fp = @fopen("ftp://$user:$pass@$address:$port".$_GET["d"]."/".$_GET["f"], "rb");
					}

                    if (!$fp)
                    {
                        include_once("lib/header.php");
                        echo "<div class=\"error\">".$text['error']." ".$text['errpassthru']."</div>";
                        include_once("lib/footer.php");
                        exit;
                    }
                }
                else
                {
                    $fp = fopen(FILEPATH."/".$_GET["f"], "w+b");
                    ftp_fget($conn, $fp, $_GET["d"]."/".$_GET["f"], FTP_BINARY);
                }

                // Send headers
                header("Cache-Control: no-cache, must-revalidate");
                header("Pragma: no-cache");
                header("Content-Type: ".getMIMEType($_GET["f"]));
                header("Content-Disposition: inline; filename=\"".$_GET["f"]."\"");
                header("Content-Length: $size");
                header("Expires: 0");

				// Send the file
                @rewind($fp);
                @fpassthru($fp) or @fclose($fp);

                // All done! Clean up.
                @fclose($fp);

                if (!PASSTHRUDOWNLOAD)
                {
                    @unlink(FILEPATH."/".$_GET["f"]);
                    @ftp_quit($conn);
                }
                exit;
            }
            else
            {
                $error	= true;
                $errmsg	= $text['errnosize'];
            }
        }
    }

    if ($downloading === true)
        exit;

    $pwd     = ftp_pwd($conn);
    $systype = ftp_systype($conn);
    $rawlist = ftp_rawlist($conn, "");
    $nlist   = rawlist2nlist($rawlist);

    @setcookie("pwd", $pwd, 0);

	require_once("lib/header.php");

	// If there were errors, display the most recent one
	if ($error)
	{
		?>
		<div class="error"><?=$text['error']?> <?=stripslashes($errmsg)?></div>
		<?
	}
	?>
	<div class="path"><?=$address.$pwd?></div>
	<?
	// If we've got a result from a custom command, let's display it
	if (isset($cmdresult))
	{
		?>
		<div class=cmdresult>
			<b><?=$text['commandresult']?></b>

			<p>
			<pre>
			<?=stripslashes(htmlspecialchars($cmdresult))?>
			</pre>
		</div>
		<?php
	}
	?>
	<div class=dirlist>
	<?
	if ($debug)
	{
		$downloaded = unserialize(base64_decode($_COOKIE["files"]));

		echo "<pre>";
		echo unserialize(base64_decode($_COOKIE["files"]));
		print_r($downloaded);
		echo "</pre>";

		echo "Systype: $systype";
		echo "<pre>".implode("\n", $nlist)."</pre>";
		echo "<pre>".implode("\n", $rawlist)."</pre>";
	}

	echo "<table cellspacing=\"1\" width=\"100%\">\n";
	echo "<tr><td class=\"tiny\" align=\"center\">".$text['del']."</td><td class=\"tiny\">".$text['name']."</td><td class=\"tiny\">".$text['size']."</td><td class=\"tiny\">".$text['date']."</td><td class=\"tiny\">".$text['permissions']."</td></tr>\n";
	echo "<tr class=\"ftpDirectory\"><td>&nbsp;</td><td><b><a href=\"ftp.php?d=".urlencode($pwd."/..")."\">..</a></b></td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td></tr>\n";

	// Print directory list
	foreach($rawlist as $item)
	{
		$symlink = islink($conn, $item);
		
		if ((isdir($item) || $symlink == 2) & (getName($item) != ".") & (getName($item) != ".."))
		{
			if ($pwd == "" | $pwd == "/")
				echo "<tr class=\"ftpDirectory\"><td width=\"20\" align=\"center\"><a href=\"ftp.php?op=del&amp;dir=".urlencode("/".getName($item))."\">x</a></td><td width=\"45%\" nowrap><b><a href=\"ftp.php?d=".urlencode("/".getName($item))."&amp;symlink=$symlink\">".getName($item)."</a></b></td><td></td><td align=right nowrap><code>".getFileDate($item)."</code></td><td align=right nowrap><code>".getPermissions($item)."</code></td></tr>\n";
			else
				echo "<tr class=\"ftpDirectory\"><td width=\"20\" align=\"center\"><a href=\"ftp.php?op=del&amp;dir=".urlencode($pwd."/".getName($item))."\">x</a></td><td width=\"45%\" nowrap><b><a href=\"ftp.php?d=".urlencode($pwd."/".getName($item))."&amp;symlink=$symlink\">".getName($item)."</a></b></td><td></td><td align=right nowrap><code>".getFileDate($item)."</code></td><td align=right nowrap><code>".getPermissions($item)."</code></td></tr>\n";
		}
	}

	// Print file list
	foreach($rawlist as $item)
	{
		$symlink = islink($conn, $item);

		if (isfile($item) || islink($conn, $item) == 1)
			echo "<tr class=\"ftpFile\"><td width=\"20\" align=\"center\"><a href=\"ftp.php?op=del&amp;file=".urlencode(getName($item))."\">x</a></td><td width=\"45%\" nowrap><a href=\"ftp.php?d=".urlencode($pwd)."&amp;f=".urlencode(getName($item))."&amp;s=".urlencode(base64_encode(getSize($item)))."&amp;symlink=$symlink\">".getName($item)."</a></td><td align=right><code>".getSize($item)."</code></td><td nowrap align=right><code>".getFileDate($item)."</code></td><td align=right nowrap><code>".getPermissions($item)."</code></td></tr>\n";
	}

	echo "</table>\n";
	?>
	</div>
	<form action="ftp.php" method="post">
	<div class="toolbar">
		<select name="op">
			<option value="cd"><?=$text['cd']?></option>
			<option value="mkdir"><?=$text['mkdir']?></option>
			<option value="logoff"><?=$text['logoff']?></option>
		</select>
		<input type="text" name="cmd" maxlength="255" size="20"/>
		<input type="submit" value="<?=$text['submit']?>"/>
	</div>
	</form>

	<form enctype="multipart/form-data" action="ftp.php" method="post">
	<div class="toolbar">
		<input type="hidden" name="MAX_FILE_SIZE" value="<?=SIZELIMIT?>"/>
		<input type="hidden" name="action" value="upload"/>
		<input name="uploadfile" type="file"/>
		<input type="submit" value="<?=$text['upload']?>"/>
	</div>
	</form>

	<?
    ftp_quit($conn);
}
include_once("lib/footer.php");
?>